/******************************************************************************/
/*                                 M o r s e                                  */
/*                                                                            */
/* * The Morse.cpp header file.                                               */
/*                                                                            */
/* Revision log                                                               */
/* -------------------------------------------------------------------------- */
/* A00 - 2016.10.22 - JOO: Original version.                                  */
/* A01 - 2019.01.01 - JOO: Rewritten.                                         */
/*                                                                            */
/*                     Copyright (C) 2019 - Sintran Data                      */
/******************************************************************************/
#ifndef MORSE_H
#define MORSE_H

#include <Arduino.h>
#include <Print.h>

//#define M_DEBUG					// Will add some trace output, only used for debugging

#define M_VERSION_MAJOR			1
#define M_VERSION_MINOR			1
#define M_VERSION_PATCH			0	// Not used
#define M_VERSION_BUILD			0	// Not used
#define M_VERSION_DATE          "2019.01.01"
#define M_VERSION_NDNO          "292012"

#define M_DEFAULT_DURATION	  100	// Default dot duration (in ms)
#define M_DEFAULT_FREQUENCY   900	// Default tone frequency (in Hz)
#define M_DEFAULT_PIN		 0xFF	// No digital output pin is defined

// All the durations are a multiple of the current dot duration.
#define M_DURATION_DOT			1	// Dot duration
#define M_DURATION_DASH			3	// Dash duration
#define M_DURATION_SIGNAL		1	// Space between morse signals
#define M_DURATION_LETTER		3	// Space between each character
#define M_DURATION_WORD			7	// Space between each word (space)

struct morseElement
{
	char morseChar;
	byte morseCode;
};

class Morse : public Print
{
public:
	Morse(int duration = M_DEFAULT_DURATION, int frequency = M_DEFAULT_FREQUENCY);
	virtual ~Morse(void) {};

	void  begin(byte pin);
	void  begin(byte pin, int duration);
	void  begin(byte pin, int duration, int frequency);
	void  dash(void);
	void  dot(void);
	int   getDuration(void);
	int   getFrequency(void);
	byte  getPin(void);
	char* string(char* str, int max, const char* txt);
	void  wait(int type);

	using   Print::write;
	virtual size_t write(uint8_t ch);

private:
	static const morseElement morseTable[];

	byte _pin;
	int  _duration;
	int  _frequency;

	byte findStartOfMorseCode(byte code);
	byte getMorseCode(char ch);
	void putChar(char* str, int* sindx, int max, char ch);
#ifdef M_DEBUG
	void trace(const char* format, ...);
#endif
};

#endif // MORSE_H
